#!/usr/bin/env python3
#
# loads amibios and calculates checksum (16 bit sequential) and value needed to validate (if checksum != 0)

import sys

try:
    bios_file = sys.argv[1]
    with open(bios_file, mode='rb') as file:
        bios_data = file.read()
except:
    print("Usage: amichksum.py path_to_rom.bin")
    sys.exit()
block_count = int(len(bios_data) / 16)
checksum = 0
for block in range(0, block_count):
    bios_block = bios_data[block*16:block*16+16]
    block_checksum = 0
    for byte_pair in range(0, 8):
        byte_pair_hex = bios_block[byte_pair*2+1:byte_pair*2+2].hex() + bios_block[byte_pair*2:byte_pair*2+1].hex()
        byte_pair_dec = int(byte_pair_hex, 16)
        block_checksum += byte_pair_dec
    block_checksum %= 65536
    checksum += block_checksum
checksum %= 65536
if checksum != 0:
    print("Checksum does not equal zero!!!")
    print("Checksum: " + str(checksum))
    two_byte_addition = int(65536-checksum).to_bytes(2, "little").hex(" ").upper()
    print("2 byte addition needed to validate checksum:", two_byte_addition)
else:
    print("Checksum equals zero")
    print("Checksum: " + str(checksum))